# Generated by Django 4.2.15 on 2024-08-25 12:09

from django.conf import settings
import django.core.files.storage
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AglService',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.CharField(max_length=255)),
                ('online', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('port', models.IntegerField(blank=True, null=True)),
                ('protocol', models.CharField(blank=True, max_length=255, null=True)),
                ('url', models.URLField(blank=True, null=True)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='CaseTemplateRuleType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='CaseTemplateRuleValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
                ('fk_value', models.CharField(blank=True, max_length=255, null=True)),
                ('numeric_value', models.FloatField(blank=True, null=True)),
                ('text_value', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='CaseTemplateRuleValueType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('distribution_type', models.CharField(choices=[('single_categorical', 'single_categorical'), ('multiple_categorical', 'multiple_categorical'), ('numeric', 'numeric')], max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='CaseTemplateType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Center',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DateValueDistribution',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('distribution_type', models.CharField(choices=[('uniform', 'Uniform'), ('normal', 'Normal')], max_length=20)),
                ('mode', models.CharField(choices=[('date', 'Date'), ('timedelta', 'Timedelta')], max_length=20)),
                ('date_min', models.DateField(blank=True, null=True)),
                ('date_max', models.DateField(blank=True, null=True)),
                ('date_mean', models.DateField(blank=True, null=True)),
                ('date_std_dev', models.IntegerField(blank=True, null=True)),
                ('timedelta_days_min', models.IntegerField(blank=True, null=True)),
                ('timedelta_days_max', models.IntegerField(blank=True, null=True)),
                ('timedelta_days_mean', models.IntegerField(blank=True, null=True)),
                ('timedelta_days_std_dev', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Disease',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DiseaseClassification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('disease', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.disease')),
            ],
        ),
        migrations.CreateModel(
            name='DiseaseClassificationChoice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('disease_classification', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.diseaseclassification')),
            ],
        ),
        migrations.CreateModel(
            name='EmissionFactor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('value', models.FloatField()),
            ],
        ),
        migrations.CreateModel(
            name='Endoscope',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('sn', models.CharField(max_length=255)),
            ],
            options={
                'verbose_name': 'Endoscope',
                'verbose_name_plural': 'Endoscopes',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='EndoscopeType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'Endoscope Type',
                'verbose_name_plural': 'Endoscope Types',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='EndoscopyProcessor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('image_width', models.IntegerField()),
                ('image_height', models.IntegerField()),
                ('endoscope_image_x', models.IntegerField()),
                ('endoscope_image_y', models.IntegerField()),
                ('endoscope_image_width', models.IntegerField()),
                ('endoscope_image_height', models.IntegerField()),
                ('examination_date_x', models.IntegerField()),
                ('examination_date_y', models.IntegerField()),
                ('examination_date_width', models.IntegerField()),
                ('examination_date_height', models.IntegerField()),
                ('examination_time_x', models.IntegerField(blank=True, null=True)),
                ('examination_time_y', models.IntegerField(blank=True, null=True)),
                ('examination_time_width', models.IntegerField(blank=True, null=True)),
                ('examination_time_height', models.IntegerField(blank=True, null=True)),
                ('patient_first_name_x', models.IntegerField()),
                ('patient_first_name_y', models.IntegerField()),
                ('patient_first_name_width', models.IntegerField()),
                ('patient_first_name_height', models.IntegerField()),
                ('patient_last_name_x', models.IntegerField()),
                ('patient_last_name_y', models.IntegerField()),
                ('patient_last_name_width', models.IntegerField()),
                ('patient_last_name_height', models.IntegerField()),
                ('patient_dob_x', models.IntegerField()),
                ('patient_dob_y', models.IntegerField()),
                ('patient_dob_width', models.IntegerField()),
                ('patient_dob_height', models.IntegerField()),
                ('endoscope_type_x', models.IntegerField(blank=True, null=True)),
                ('endoscope_type_y', models.IntegerField(blank=True, null=True)),
                ('endoscope_type_width', models.IntegerField(blank=True, null=True)),
                ('endoscope_type_height', models.IntegerField(blank=True, null=True)),
                ('endoscope_sn_x', models.IntegerField(blank=True, null=True)),
                ('endoscope_sn_y', models.IntegerField(blank=True, null=True)),
                ('endoscope_sn_width', models.IntegerField(blank=True, null=True)),
                ('endoscope_sn_height', models.IntegerField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Examination',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('date', models.DateField(blank=True, null=True)),
                ('time', models.TimeField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Examination',
                'verbose_name_plural': 'Examinations',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ExaminationType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Examiner',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=255)),
                ('last_name', models.CharField(max_length=255)),
                ('dob', models.DateField(blank=True, null=True, verbose_name='Date of Birth')),
                ('email', models.EmailField(blank=True, max_length=255, null=True)),
                ('phone', models.CharField(blank=True, max_length=255, null=True)),
                ('is_real_person', models.BooleanField(default=True)),
                ('center', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FFMpegMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('duration', models.FloatField(blank=True, null=True)),
                ('width', models.IntegerField(blank=True, null=True)),
                ('height', models.IntegerField(blank=True, null=True)),
                ('frame_rate', models.FloatField(blank=True, null=True)),
                ('video_codec', models.CharField(blank=True, max_length=50, null=True)),
                ('audio_codec', models.CharField(blank=True, max_length=50, null=True)),
                ('audio_channels', models.IntegerField(blank=True, null=True)),
                ('audio_sample_rate', models.IntegerField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='FirstName',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Frame',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('frame_number', models.IntegerField()),
                ('image', models.ImageField(upload_to='frames')),
                ('suffix', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Gender',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('abbreviation', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='InformationSource',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('url', models.URLField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('date', models.DateField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Label',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='LabelSet',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
                ('version', models.IntegerField()),
                ('labels', models.ManyToManyField(related_name='labels', to='endoreg_db.label')),
            ],
        ),
        migrations.CreateModel(
            name='LabelType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='LabValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('abbreviation', models.CharField(blank=True, max_length=10, null=True)),
                ('numeric_precision', models.IntegerField(default=3)),
                ('default_normal_range', models.JSONField(blank=True, null=True)),
                ('normal_range_age_dependent', models.BooleanField(default=False)),
                ('normal_range_gender_dependent', models.BooleanField(default=False)),
                ('normal_range_special_case', models.BooleanField(default=False)),
                ('default_date_value_distribution', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='default_date_value_distribution', to='endoreg_db.datevaluedistribution')),
            ],
        ),
        migrations.CreateModel(
            name='LastName',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='LegacyImage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(upload_to='legacy_images')),
                ('suffix', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='LegacyVideo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('video_hash', models.CharField(max_length=255, unique=True)),
                ('date', models.DateField(blank=True, null=True)),
                ('suffix', models.CharField(max_length=255)),
                ('fps', models.FloatField()),
                ('duration', models.FloatField()),
                ('width', models.IntegerField()),
                ('height', models.IntegerField()),
                ('endoscope_image_x', models.IntegerField(blank=True, null=True)),
                ('endoscope_image_y', models.IntegerField(blank=True, null=True)),
                ('endoscope_image_width', models.IntegerField(blank=True, null=True)),
                ('endoscope_image_height', models.IntegerField(blank=True, null=True)),
                ('frames_extracted', models.BooleanField(default=False)),
                ('meta', models.JSONField(blank=True, null=True)),
                ('file', models.FileField(blank=True, null=True, upload_to='legacy_videos')),
                ('center', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('endoscopy_processor', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.endoscopyprocessor')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LogType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.CharField(max_length=255)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Material',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('emission_factor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.emissionfactor')),
            ],
        ),
        migrations.CreateModel(
            name='Medication',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('adapt_to_renal_function', models.BooleanField(default=False)),
                ('adapt_to_hepatic_function', models.BooleanField(default=False)),
                ('adapt_to_indication', models.BooleanField(default=False)),
                ('adapt_to_age', models.BooleanField(default=False)),
                ('adapt_to_weight', models.BooleanField(default=False)),
                ('adapt_to_risk', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='MedicationIndication',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('classification_choices', models.ManyToManyField(to='endoreg_db.diseaseclassificationchoice')),
                ('diseases', models.ManyToManyField(to='endoreg_db.disease')),
                ('events', models.ManyToManyField(to='endoreg_db.event')),
            ],
        ),
        migrations.CreateModel(
            name='MedicationIndicationType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='MedicationIntakeTime',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('repeats', models.CharField(default='daily', max_length=20)),
                ('time', models.TimeField()),
            ],
        ),
        migrations.CreateModel(
            name='MedicationSchedule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('therapy_duration_d', models.FloatField(blank=True, null=True)),
                ('dose', models.FloatField()),
                ('intake_times', models.ManyToManyField(to='endoreg_db.medicationintaketime')),
                ('medication', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.medication')),
            ],
        ),
        migrations.CreateModel(
            name='ModelMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('version', models.CharField(max_length=255)),
                ('weights', models.FileField(upload_to='weights/')),
                ('description', models.TextField(blank=True, null=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('labelset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='models', to='endoreg_db.labelset')),
            ],
        ),
        migrations.CreateModel(
            name='ModelType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='MultipleCategoricalValueDistribution',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('categories', models.JSONField()),
                ('min_count', models.IntegerField()),
                ('max_count', models.IntegerField()),
                ('count_distribution_type', models.CharField(choices=[('uniform', 'Uniform'), ('normal', 'Normal')], max_length=20)),
                ('count_mean', models.FloatField(blank=True, null=True)),
                ('count_std_dev', models.FloatField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NetworkDevice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('ip', models.GenericIPAddressField(blank=True, null=True)),
                ('description', models.CharField(max_length=255)),
                ('online', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'network_devices',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='NetworkDeviceType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.CharField(max_length=255)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'network_device_types',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='NumericValueDistribution',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('distribution_type', models.CharField(choices=[('uniform', 'Uniform'), ('normal', 'Normal'), ('skewed_normal', 'Skewed Normal')], max_length=20)),
                ('min_value', models.FloatField()),
                ('max_value', models.FloatField()),
                ('mean', models.FloatField(blank=True, null=True)),
                ('std_dev', models.FloatField(blank=True, null=True)),
                ('skewness', models.FloatField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Patient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=255)),
                ('last_name', models.CharField(max_length=255)),
                ('dob', models.DateField(blank=True, null=True, verbose_name='Date of Birth')),
                ('email', models.EmailField(blank=True, max_length=255, null=True)),
                ('phone', models.CharField(blank=True, max_length=255, null=True)),
                ('is_real_person', models.BooleanField(default=True)),
                ('center', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('gender', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.gender')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PatientLabSample',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField()),
                ('patient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='lab_samples', to='endoreg_db.patient')),
            ],
        ),
        migrations.CreateModel(
            name='PatientLabSampleType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='PatientMedication',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dosage', models.JSONField()),
                ('active', models.BooleanField(default=True)),
                ('medication_indication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_medications', to='endoreg_db.medicationindication')),
                ('medication_schedules', models.ManyToManyField(to='endoreg_db.medicationschedule')),
                ('patient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patient')),
            ],
            options={
                'verbose_name': 'Patient Medication',
                'verbose_name_plural': 'Patient Medications',
            },
        ),
        migrations.CreateModel(
            name='PdfType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ProductGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Profession',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='QuizAnswerType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='QuizQuestionType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ReportReaderFlag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('value', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Resource',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Rule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('attribute_key', models.CharField(max_length=255)),
            ],
            options={
                'verbose_name': 'Rule',
                'verbose_name_plural': 'Rules',
            },
        ),
        migrations.CreateModel(
            name='RuleAttributeDType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='RuleType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'Rule Type',
                'verbose_name_plural': 'Rule Types',
            },
        ),
        migrations.CreateModel(
            name='SensitiveMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('examination_date', models.DateField(blank=True, null=True)),
                ('patient_first_name', models.CharField(blank=True, max_length=255, null=True)),
                ('patient_last_name', models.CharField(blank=True, max_length=255, null=True)),
                ('patient_dob', models.DateField(blank=True, null=True)),
                ('endoscope_type', models.CharField(blank=True, max_length=255, null=True)),
                ('endoscope_sn', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='SingleCategoricalValueDistribution',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('categories', models.JSONField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TtoQuestionnaire',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('patient_name', models.CharField(max_length=255, verbose_name='Identifikation des Patienten (Name)')),
                ('birth_date', models.DateField(verbose_name='Geburtsdatum')),
                ('emergency_patient', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Notfallpatient/kürzlich untersuchter Patient (Verzicht auf Team-Time-Out möglich)')),
                ('consent_signed', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Einverständniserklärung unterschrieben (Arzt, Patient)')),
                ('documents_present', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Alle Dokumente liegen vor (Labor, Befunde, etc.)')),
                ('communication_possible', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Kommunikation mit Patient möglich')),
                ('work_incapacity_certificate', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Arbeitsunfähigkeitsbescheinigung')),
                ('pregnancy', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Schwangerschaft')),
                ('asa_classification_checked', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='ASA-Klassifikation/Komorbidität geprüft')),
                ('previous_anesthesia_complications', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Komplikationen bei bisherigen Narkosen?')),
                ('last_meal_over_6_hours_ago', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Zeitpunkt letzte Mahlzeit > 6 Stunden')),
                ('allergies', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Allergien, welche?')),
                ('outpatient_accompaniment_post_sedation', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Nur bei ambulanter Vorstellung: Begleitung nach Sedierung?')),
                ('dental_prosthesis', models.BooleanField(default=False, verbose_name='Zahnprothese')),
                ('glasses', models.BooleanField(default=False, verbose_name='Brille')),
                ('implants', models.BooleanField(default=False, verbose_name='Implantate')),
                ('hearing_aids', models.BooleanField(default=False, verbose_name='Hörgeräte')),
                ('anticoagulants_ass', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Antikoagulation, ASS')),
                ('blood_pressure_medication', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Blutdruckmedikamente')),
                ('glaucoma', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Glaukom')),
                ('metal_implants', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Metallimplantate')),
                ('pacemaker_icd', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Herzschrittmacher/ICD')),
                ('copd', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='COPD')),
                ('liver_cirrhosis', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Leberzirrhose')),
                ('ibd', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='CED (Chronisch entzündliche Darmerkrankungen)')),
                ('radiation', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Bestrahlung')),
                ('surgeries', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='OP´s')),
                ('team_introduction', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Teamvorstellung mit Name und Aufgabe')),
                ('instruments_available', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Notwendige Instrumente vorhanden?')),
                ('monitoring_medications_equipment_checked', models.CharField(choices=[('yes', 'Ja'), ('no', 'Nein')], max_length=3, verbose_name='Monitoring, Medikamente, Equipment zum Atemwegsmanagement zur Verfügung und überprüft?')),
                ('complete_documentation_inclusive_care_notes', models.TextField(verbose_name='Vollständige Dokumentation inklusive Hinweise für Nachsorge')),
                ('notes', models.TextField(default='Keine Bemerkungen', verbose_name='Bemerkungen')),
                ('specimens_secured', models.BooleanField(default=False, verbose_name='Histologische Proben gesichert')),
                ('patient_condition_documented', models.TextField(verbose_name='Patientenzustand dokumentiert (je nach Ausgangszustand)')),
            ],
        ),
        migrations.CreateModel(
            name='Unit',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('description', models.CharField(blank=True, max_length=100, null=True)),
                ('abbreviation', models.CharField(blank=True, max_length=25, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(blank=True, null=True, upload_to='raw_videos')),
                ('video_hash', models.CharField(max_length=255, unique=True)),
                ('date', models.DateField(blank=True, null=True)),
                ('suffix', models.CharField(max_length=255)),
                ('fps', models.FloatField()),
                ('duration', models.FloatField()),
                ('width', models.IntegerField()),
                ('height', models.IntegerField()),
                ('endoscope_image_x', models.IntegerField(blank=True, null=True)),
                ('endoscope_image_y', models.IntegerField(blank=True, null=True)),
                ('endoscope_image_width', models.IntegerField(blank=True, null=True)),
                ('endoscope_image_height', models.IntegerField(blank=True, null=True)),
                ('frames_extracted', models.BooleanField(default=False)),
                ('meta', models.JSONField(blank=True, null=True)),
                ('center', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('endoscopy_processor', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.endoscopyprocessor')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VideoImportMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('video_anonymized', models.BooleanField(default=False)),
                ('video_patient_data_detected', models.BooleanField(default=False)),
                ('outside_detected', models.BooleanField(default=False)),
                ('patient_data_removed', models.BooleanField(default=False)),
                ('outside_removed', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Waste',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='VideoPredictionMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
                ('prediction_array', models.BinaryField(blank=True, null=True)),
                ('model_meta', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.modelmeta')),
                ('video', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='video_prediction_meta', to='endoreg_db.video')),
            ],
            options={
                'abstract': False,
                'unique_together': {('model_meta', 'video')},
            },
        ),
        migrations.CreateModel(
            name='VideoMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('endoscope', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.endoscope')),
                ('ffmpeg_meta', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.ffmpegmeta')),
                ('import_meta', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.videoimportmeta')),
                ('processor', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.endoscopyprocessor')),
            ],
        ),
        migrations.AddField(
            model_name='video',
            name='import_meta',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.videoimportmeta'),
        ),
        migrations.AddField(
            model_name='video',
            name='patient',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patient'),
        ),
        migrations.CreateModel(
            name='TransportRoute',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('distance', models.FloatField()),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('emission_factor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.emissionfactor')),
                ('unit', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.unit')),
            ],
        ),
        migrations.CreateModel(
            name='Ruleset',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=255, null=True)),
                ('name_en', models.CharField(blank=True, max_length=255, null=True)),
                ('rules', models.ManyToManyField(to='endoreg_db.rule')),
            ],
        ),
        migrations.AddField(
            model_name='rule',
            name='attribute_dtype',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.ruleattributedtype'),
        ),
        migrations.AddField(
            model_name='rule',
            name='rule_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.ruletype'),
        ),
        migrations.CreateModel(
            name='ReportReaderConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('locale', models.CharField(default='de_DE', max_length=10)),
                ('text_date_format', models.CharField(default='%d.%m.%Y', max_length=10)),
                ('cut_off_above', models.ManyToManyField(related_name='report_reader_configs_cut_off_above', to='endoreg_db.reportreaderflag')),
                ('cut_off_below', models.ManyToManyField(related_name='report_reader_configs_cut_off_below', to='endoreg_db.reportreaderflag')),
                ('endoscope_info_line_flag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='report_reader_configs_endoscope_info_line', to='endoreg_db.reportreaderflag')),
                ('examiner_info_line_flag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='report_reader_configs_examiner_info_line', to='endoreg_db.reportreaderflag')),
                ('first_names', models.ManyToManyField(related_name='report_reader_configs', to='endoreg_db.firstname')),
                ('last_names', models.ManyToManyField(related_name='report_reader_configs', to='endoreg_db.lastname')),
                ('patient_info_line_flag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='report_reader_configs_patient_info_line', to='endoreg_db.reportreaderflag')),
            ],
        ),
        migrations.CreateModel(
            name='ReportFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('pdf', models.FileField(blank=True, null=True, upload_to='raw_report_pdfs')),
                ('pdf_hash', models.CharField(max_length=255, unique=True)),
                ('meta', models.JSONField(blank=True, null=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('text_anonymized', models.TextField(blank=True, null=True)),
                ('date', models.DateField(blank=True, null=True)),
                ('time', models.TimeField(blank=True, null=True)),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('examiner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.examiner')),
                ('patient', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patient')),
            ],
        ),
        migrations.CreateModel(
            name='ReferenceProduct',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('emission_factor_package', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='reference_product_package', to='endoreg_db.emissionfactor')),
                ('emission_factor_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='reference_product_product', to='endoreg_db.emissionfactor')),
                ('emission_factor_total', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.emissionfactor')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.product')),
                ('product_group', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='reference_product', to='endoreg_db.productgroup')),
            ],
        ),
        migrations.CreateModel(
            name='RawVideoFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField()),
                ('file', models.FileField(upload_to='raw_data/')),
                ('original_file_name', models.CharField(max_length=255)),
                ('video_hash', models.CharField(max_length=255, unique=True)),
                ('uploaded_at', models.DateTimeField(auto_now_add=True)),
                ('state_frames_required', models.BooleanField(default=True)),
                ('state_frames_extracted', models.BooleanField(default=False)),
                ('state_initial_prediction_required', models.BooleanField(default=True)),
                ('state_initial_prediction_completed', models.BooleanField(default=False)),
                ('state_initial_prediction_import_required', models.BooleanField(default=True)),
                ('state_initial_prediction_import_completed', models.BooleanField(default=False)),
                ('state_ocr_required', models.BooleanField(default=True)),
                ('state_ocr_completed', models.BooleanField(default=False)),
                ('state_outside_validated', models.BooleanField(default=False)),
                ('state_ocr_result_validated', models.BooleanField(default=False)),
                ('state_sensitive_data_retrieved', models.BooleanField(default=False)),
                ('state_histology_required', models.BooleanField(blank=True, null=True)),
                ('state_histology_available', models.BooleanField(default=False)),
                ('state_follow_up_intervention_required', models.BooleanField(blank=True, null=True)),
                ('state_follow_up_intervention_available', models.BooleanField(default=False)),
                ('state_dataset_complete', models.BooleanField(default=False)),
                ('state_anonym_video_required', models.BooleanField(default=True)),
                ('state_anonym_video_performed', models.BooleanField(default=False)),
                ('state_original_reports_deleted', models.BooleanField(default=False)),
                ('state_original_video_deleted', models.BooleanField(default=False)),
                ('state_finalized', models.BooleanField(default=False)),
                ('frame_dir', models.CharField(max_length=255)),
                ('prediction_dir', models.CharField(max_length=255)),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('processor', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.endoscopyprocessor')),
                ('sensitive_meta', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.sensitivemeta')),
                ('video_meta', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.videometa')),
            ],
        ),
        migrations.CreateModel(
            name='RawPdfFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(storage=django.core.files.storage.FileSystemStorage(location='/home/agl-admin/agl-monitor/agl_monitor/agl_monitor/erc_data/raw_pdf'), upload_to='raw_pdf/', validators=[django.core.validators.FileExtensionValidator(allowed_extensions=['pdf'])])),
                ('pdf_hash', models.CharField(max_length=255, unique=True)),
                ('state_report_processing_required', models.BooleanField(default=True)),
                ('state_report_processed', models.BooleanField(default=False)),
                ('text', models.TextField(blank=True, null=True)),
                ('anonymized_text', models.TextField(blank=True, null=True)),
                ('raw_meta', models.JSONField(blank=True, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('pdf_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.pdftype')),
                ('sensitive_meta', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='raw_pdf_file', to='endoreg_db.sensitivemeta')),
            ],
        ),
        migrations.CreateModel(
            name='QuizQuestion',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('description', models.TextField(blank=True, null=True)),
                ('question_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.quizquestiontype')),
            ],
        ),
        migrations.CreateModel(
            name='QuizAnswer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text_value', models.CharField(blank=True, max_length=255, null=True)),
                ('number_value', models.FloatField(blank=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True)),
                ('answer_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.quizanswertype')),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.quizquestion')),
            ],
        ),
        migrations.CreateModel(
            name='ProductWeight',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, null=True)),
                ('measured', models.FloatField(null=True)),
                ('verified', models.FloatField(null=True)),
                ('manufacturer', models.FloatField(null=True)),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.product')),
                ('unit', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.unit')),
            ],
        ),
        migrations.CreateModel(
            name='ProductMaterial',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('component', models.CharField(max_length=255)),
                ('quantity', models.FloatField()),
                ('material', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.material')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='product_materials', to='endoreg_db.product')),
                ('unit', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.unit')),
            ],
        ),
        migrations.AddField(
            model_name='product',
            name='product_group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.productgroup'),
        ),
        migrations.AddField(
            model_name='product',
            name='transport_route',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.transportroute'),
        ),
        migrations.CreateModel(
            name='PortalUserInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('works_in_endoscopy', models.BooleanField(blank=True, null=True)),
                ('profession', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.profession')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='pdftype',
            name='cut_off_above_lines',
            field=models.ManyToManyField(related_name='pdf_type_cut_off_above_lines', to='endoreg_db.reportreaderflag'),
        ),
        migrations.AddField(
            model_name='pdftype',
            name='cut_off_below_lines',
            field=models.ManyToManyField(related_name='pdf_type_cut_off_below_lines', to='endoreg_db.reportreaderflag'),
        ),
        migrations.AddField(
            model_name='pdftype',
            name='endoscope_info_line',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pdf_type_endoscopy_info_line', to='endoreg_db.reportreaderflag'),
        ),
        migrations.AddField(
            model_name='pdftype',
            name='examiner_info_line',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pdf_type_examiner_info_line', to='endoreg_db.reportreaderflag'),
        ),
        migrations.AddField(
            model_name='pdftype',
            name='patient_info_line',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pdf_type_patient_info_line', to='endoreg_db.reportreaderflag'),
        ),
        migrations.CreateModel(
            name='PdfMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField()),
                ('time', models.TimeField()),
                ('pdf_hash', models.CharField(max_length=255, unique=True)),
                ('pdf_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.pdftype')),
            ],
        ),
        migrations.CreateModel(
            name='PatientMedicationSchedule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('medication', models.ManyToManyField(related_name='patient_medication_schedules', to='endoreg_db.patientmedication')),
                ('patient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patient')),
            ],
        ),
        migrations.AddField(
            model_name='patientmedication',
            name='unit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.unit'),
        ),
        migrations.CreateModel(
            name='PatientLabValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.FloatField(blank=True, null=True)),
                ('value_str', models.CharField(blank=True, max_length=255, null=True)),
                ('datetime', models.DateTimeField(auto_now_add=True)),
                ('normal_range', models.JSONField(default=dict)),
                ('lab_value', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.labvalue')),
                ('patient', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='lab_values', to='endoreg_db.patient')),
                ('sample', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='values', to='endoreg_db.patientlabsample')),
                ('unit', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.unit')),
            ],
        ),
        migrations.AddField(
            model_name='patientlabsample',
            name='sample_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patientlabsampletype'),
        ),
        migrations.CreateModel(
            name='PatientExamination',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('examination', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.examination')),
                ('patient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='patient_examinations', to='endoreg_db.patient')),
                ('report_file', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_examination', to='endoreg_db.reportfile')),
                ('video', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_examination', to='endoreg_db.video')),
            ],
            options={
                'verbose_name': 'Patient Examination',
                'verbose_name_plural': 'Patient Examinations',
                'ordering': ['patient', 'examination'],
            },
        ),
        migrations.CreateModel(
            name='PatientEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_start', models.DateField()),
                ('date_end', models.DateField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.event')),
                ('patient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patient')),
            ],
        ),
        migrations.CreateModel(
            name='PatientDisease',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_date', models.DateField(blank=True, null=True)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('classification_choices', models.ManyToManyField(to='endoreg_db.diseaseclassificationchoice')),
                ('disease', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.disease')),
                ('patient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patient')),
            ],
        ),
        migrations.CreateModel(
            name='NetworkDeviceLogEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('ip_address', models.GenericIPAddressField(blank=True, null=True)),
                ('user_agent', models.CharField(blank=True, max_length=256, null=True)),
                ('message', models.TextField()),
                ('json_content', models.JSONField(blank=True, null=True)),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.networkdevice')),
                ('log_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.logtype')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Network Device Log Entry',
                'verbose_name_plural': 'Network Device Log Entries',
            },
        ),
        migrations.AddField(
            model_name='networkdevice',
            name='device_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.networkdevicetype'),
        ),
        migrations.AddField(
            model_name='modelmeta',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='models', to='endoreg_db.modeltype'),
        ),
        migrations.AddField(
            model_name='medicationschedule',
            name='unit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.unit'),
        ),
        migrations.AddField(
            model_name='medicationindication',
            name='indication_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='medication_indications', to='endoreg_db.medicationindicationtype'),
        ),
        migrations.AddField(
            model_name='medicationindication',
            name='medication_schedules',
            field=models.ManyToManyField(to='endoreg_db.medicationschedule'),
        ),
        migrations.AddField(
            model_name='medicationindication',
            name='sources',
            field=models.ManyToManyField(to='endoreg_db.informationsource'),
        ),
        migrations.AddField(
            model_name='medication',
            name='default_unit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.unit'),
        ),
        migrations.CreateModel(
            name='LegacyVideoPredictionMeta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
                ('prediction_array', models.BinaryField(blank=True, null=True)),
                ('model_meta', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.modelmeta')),
                ('video', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='video_prediction_meta', to='endoreg_db.legacyvideo')),
            ],
            options={
                'abstract': False,
                'unique_together': {('model_meta', 'video')},
            },
        ),
        migrations.AddField(
            model_name='legacyvideo',
            name='patient',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.patient'),
        ),
        migrations.CreateModel(
            name='LegacyLabelVideoSegment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_frame_number', models.IntegerField()),
                ('end_frame_number', models.IntegerField()),
                ('label', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.label')),
                ('prediction_meta', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='video_segments', to='endoreg_db.legacyvideopredictionmeta')),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.informationsource')),
                ('video', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.legacyvideo')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LegacyFrame',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('frame_number', models.IntegerField()),
                ('image', models.ImageField(blank=True, null=True, upload_to='legacy_frames')),
                ('suffix', models.CharField(max_length=255)),
                ('video', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='frames', to='endoreg_db.legacyvideo')),
            ],
        ),
        migrations.CreateModel(
            name='LegacyBinaryClassificationAnnotationTask',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_finished', models.BooleanField(default=False)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_finished', models.DateTimeField(blank=True, null=True)),
                ('image_path', models.CharField(blank=True, max_length=255, null=True)),
                ('labelstudio_project_id', models.IntegerField(blank=True, null=True)),
                ('labelstudio_task_id', models.IntegerField(blank=True, null=True)),
                ('image_type', models.CharField(default='legacy', max_length=255)),
                ('frame', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='binary_classification_annotation_tasks', to='endoreg_db.legacyframe')),
                ('label', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.label')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='labvalue',
            name='default_multiple_categorical_value_distribution',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='default_multiple_categorical_value_distribution', to='endoreg_db.multiplecategoricalvaluedistribution'),
        ),
        migrations.AddField(
            model_name='labvalue',
            name='default_numerical_value_distribution',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='default_numerical_value_distribution', to='endoreg_db.numericvaluedistribution'),
        ),
        migrations.AddField(
            model_name='labvalue',
            name='default_single_categorical_value_distribution',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='default_single_categorical_value_distribution', to='endoreg_db.singlecategoricalvaluedistribution'),
        ),
        migrations.AddField(
            model_name='labvalue',
            name='default_unit',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.unit'),
        ),
        migrations.CreateModel(
            name='LabelVideoSegment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_frame_number', models.IntegerField()),
                ('end_frame_number', models.IntegerField()),
                ('label', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.label')),
                ('prediction_meta', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='video_segments', to='endoreg_db.videopredictionmeta')),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.informationsource')),
                ('video', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.video')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='label',
            name='label_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='labels', to='endoreg_db.labeltype'),
        ),
        migrations.CreateModel(
            name='ImageClassificationPrediction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.BooleanField()),
                ('confidence', models.FloatField()),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('frame', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_predictions', to='endoreg_db.frame')),
                ('label', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_predictions', to='endoreg_db.label')),
                ('legacy_frame', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_predictions', to='endoreg_db.legacyframe')),
                ('legacy_image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_predictions', to='endoreg_db.legacyimage')),
                ('model_meta', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_predictions', to='endoreg_db.modelmeta')),
            ],
        ),
        migrations.CreateModel(
            name='ImageClassificationAnnotation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.BooleanField()),
                ('annotator', models.CharField(max_length=255)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
                ('frame', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_annotations', to='endoreg_db.frame')),
                ('label', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_annotations', to='endoreg_db.label')),
                ('legacy_frame', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_annotations', to='endoreg_db.legacyframe')),
                ('legacy_image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_classification_annotations', to='endoreg_db.legacyimage')),
            ],
        ),
        migrations.AddField(
            model_name='frame',
            name='video',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='frames', to='endoreg_db.video'),
        ),
        migrations.AddField(
            model_name='examiner',
            name='gender',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.gender'),
        ),
        migrations.CreateModel(
            name='ExaminationTimeType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('examinations', models.ManyToManyField(blank=True, to='endoreg_db.examination')),
            ],
            options={
                'verbose_name': 'Examination Time Type',
                'verbose_name_plural': 'Examination Time Types',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ExaminationTime',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('name_de', models.CharField(blank=True, max_length=100, null=True)),
                ('name_en', models.CharField(blank=True, max_length=100, null=True)),
                ('start_time', models.TimeField(blank=True, null=True)),
                ('end_time', models.TimeField(blank=True, null=True)),
                ('time_types', models.ManyToManyField(blank=True, to='endoreg_db.examinationtimetype')),
            ],
            options={
                'verbose_name': 'Examination Time',
                'verbose_name_plural': 'Examination Times',
                'ordering': ['name'],
            },
        ),
        migrations.AddField(
            model_name='examination',
            name='examination_types',
            field=models.ManyToManyField(blank=True, to='endoreg_db.examinationtype'),
        ),
        migrations.AddField(
            model_name='emissionfactor',
            name='unit',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.unit'),
        ),
        migrations.CreateModel(
            name='CenterWaste',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField()),
                ('quantity', models.FloatField()),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('emission_factor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.emissionfactor')),
                ('unit', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.unit')),
                ('waste', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.waste')),
            ],
        ),
        migrations.CreateModel(
            name='CenterResource',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, null=True)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('quantity', models.FloatField()),
                ('year', models.IntegerField()),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('resource', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.resource')),
                ('transport_emission_factor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='center_resource_transport_emission_factor', to='endoreg_db.emissionfactor')),
                ('unit', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.unit')),
                ('use_emission_factor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='center_resource_use_emission_factor', to='endoreg_db.emissionfactor')),
            ],
        ),
        migrations.CreateModel(
            name='CenterProduct',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_used', models.DateField()),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.center')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.product')),
            ],
        ),
        migrations.AddField(
            model_name='center',
            name='first_names',
            field=models.ManyToManyField(related_name='centers', to='endoreg_db.firstname'),
        ),
        migrations.AddField(
            model_name='center',
            name='last_names',
            field=models.ManyToManyField(related_name='centers', to='endoreg_db.lastname'),
        ),
        migrations.CreateModel(
            name='CaseTemplateRule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('name_de', models.CharField(max_length=255, null=True)),
                ('name_en', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('parent_model', models.CharField(blank=True, max_length=255, null=True)),
                ('parent_field', models.CharField(blank=True, max_length=255, null=True)),
                ('target_field', models.CharField(blank=True, max_length=255, null=True)),
                ('target_model', models.CharField(blank=True, max_length=255, null=True)),
                ('rule_values', models.JSONField(blank=True, null=True)),
                ('extra_parameters', models.JSONField(blank=True, null=True)),
                ('chained_rules', models.ManyToManyField(related_name='calling_rules', to='endoreg_db.casetemplaterule')),
                ('date_value_distribution', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.datevaluedistribution')),
                ('multiple_categorical_value_distribution', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.multiplecategoricalvaluedistribution')),
                ('numerical_value_distribution', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.numericvaluedistribution')),
                ('rule_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.casetemplateruletype')),
                ('single_categorical_value_distribution', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.singlecategoricalvaluedistribution')),
                ('value_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.casetemplaterulevaluetype')),
            ],
        ),
        migrations.CreateModel(
            name='CaseTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('rules', models.ManyToManyField(to='endoreg_db.casetemplaterule')),
                ('secondary_rules', models.ManyToManyField(related_name='secondary_rules', to='endoreg_db.casetemplaterule')),
                ('template_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='case_templates', to='endoreg_db.casetemplatetype')),
            ],
        ),
        migrations.CreateModel(
            name='BinaryClassificationAnnotationTask',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_finished', models.BooleanField(default=False)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_finished', models.DateTimeField(blank=True, null=True)),
                ('image_path', models.CharField(blank=True, max_length=255, null=True)),
                ('labelstudio_project_id', models.IntegerField(blank=True, null=True)),
                ('labelstudio_task_id', models.IntegerField(blank=True, null=True)),
                ('image_type', models.CharField(default='frame', max_length=255)),
                ('frame', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='binary_classification_annotation_tasks', to='endoreg_db.frame')),
                ('label', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.label')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AglServiceLogEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('ip_address', models.GenericIPAddressField(blank=True, null=True)),
                ('user_agent', models.CharField(blank=True, max_length=256, null=True)),
                ('message', models.TextField()),
                ('json_content', models.JSONField(blank=True, null=True)),
                ('device', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.networkdevice')),
                ('log_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.logtype')),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='endoreg_db.aglservice')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'AGL Service Log Entry',
                'verbose_name_plural': 'AGL Service Log Entries',
            },
        ),
        migrations.AddField(
            model_name='aglservice',
            name='devices',
            field=models.ManyToManyField(related_name='services', to='endoreg_db.networkdevice'),
        ),
        migrations.CreateModel(
            name='ActiveModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('model_meta', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.modelmeta')),
            ],
        ),
        migrations.AddIndex(
            model_name='legacyframe',
            index=models.Index(fields=['video', 'frame_number'], name='endoreg_db__video_i_e10f1c_idx'),
        ),
        migrations.AlterUniqueTogether(
            name='legacyframe',
            unique_together={('video', 'frame_number')},
        ),
        migrations.AlterUniqueTogether(
            name='imageclassificationprediction',
            unique_together={('label', 'frame', 'model_meta')},
        ),
        migrations.AddIndex(
            model_name='frame',
            index=models.Index(fields=['video', 'frame_number'], name='endoreg_db__video_i_56cc44_idx'),
        ),
        migrations.AlterUniqueTogether(
            name='frame',
            unique_together={('video', 'frame_number')},
        ),
    ]



from django.db import models

class ProductMaterial(models.Model):
    component = models.CharField(max_length=255)
    material = models.ForeignKey("Material", on_delete=models.CASCADE)
    product = models.ForeignKey("Product", on_delete=models.CASCADE, related_name="product_materials")
    unit = models.ForeignKey("Unit", on_delete=models.CASCADE)
    quantity = models.FloatField()

    def get_emission(self):
        from ..emission import EmissionFactor
        emission_factor:EmissionFactor = self.material.emission_factor
        if emission_factor is None:
            raise Exception("No emission factor for material " + self.material.name + " found.")
        
        # make sure product_material.unit is the same as emission_factor.unit
        if self.unit != emission_factor.unit:
            raise Exception("Unit mismatch: " + self.unit.name + " != " + emission_factor.unit.name)
        
        emmision_value = emission_factor.value * self.quantity
        emission_unit = emission_factor.unit
        return emmision_value, emission_unit
        




from django.db import models
from rest_framework import serializers
from ..label import Label
from ..data_file.video_segment import LegacyLabelVideoSegment
from .image_classification import ImageClassificationAnnotation

ANNOTATION_PER_S_THRESHOLD = 2

def clear_finished_legacy_tasks():
    # fetch all BinaryClassificationAnnotationTasks that are finished and delete them
    tasks = LegacyBinaryClassificationAnnotationTask.objects.filter(finished=True)

    # delete tasks with a bulk operation
    tasks.delete()

def get_legacy_binary_classification_annotation_tasks_by_label(label:Label, n:int=100, legacy=False):
    clear_finished_legacy_tasks()

    if legacy:
        # fetch all LegacyLabelVideoSegments with the given label
        _segments = LegacyLabelVideoSegment.objects.filter(label=label)
        frames_for_tasks = []

        for segment in _segments:
            # check if the segment has already been annotated
            annotations = list(ImageClassificationAnnotation.objects.filter(legacy_frame__in=segment.get_frames(), label=label))
            segment_len_in_s = segment.get_segment_len_in_s()

            target_annotation_number = segment_len_in_s * ANNOTATION_PER_S_THRESHOLD

            if len(annotations) < target_annotation_number:
                get_frame_number = int(target_annotation_number - len(annotations))
                frames = segment.get_frames_without_annotation(get_frame_number)
                frames_for_tasks.extend(frames)

            if len(frames_for_tasks) >= n:
                break

        # create tasks
        tasks = []
        for frame in frames_for_tasks:

            # get_or_create task
            task, created = LegacyBinaryClassificationAnnotationTask.objects.get_or_create(
                label=label,
                image_path=frame.image.path,
                frame_id=frame.pk,
            )
        

class AbstractBinaryClassificationAnnotationTask(models.Model):
    label = models.ForeignKey("Label", on_delete=models.CASCADE)
    is_finished = models.BooleanField(default=False)
    date_created = models.DateTimeField(auto_now_add=True)
    date_finished = models.DateTimeField(blank=True, null=True)
    image_path = models.CharField(max_length=255, blank=True, null=True)
    image_type = models.CharField(max_length=255, blank=True, null=True)
    frame_id = models.IntegerField(blank=True, null=True)
    labelstudio_project_id = models.IntegerField(blank=True, null=True)
    labelstudio_task_id = models.IntegerField(blank=True, null=True)

    class Meta:
        abstract = True

class BinaryClassificationAnnotationTask(AbstractBinaryClassificationAnnotationTask):
    frame = models.ForeignKey("Frame", on_delete=models.CASCADE, related_name="binary_classification_annotation_tasks")
    image_type = models.CharField(max_length=255, default="frame")

    def get_frame(self):
        return self.video_segment.get_frame_by_id(self.frame_id)

class LegacyBinaryClassificationAnnotationTask(AbstractBinaryClassificationAnnotationTask):
    frame = models.ForeignKey("LegacyFrame", on_delete=models.CASCADE, related_name="binary_classification_annotation_tasks")
    image_type = models.CharField(max_length=255, default="legacy")

    def get_frame(self):
        return self.frame






from django.db import models

# Serializer located in serializers/examination.py
class PatientExamination(models.Model):
    patient = models.ForeignKey('Patient', on_delete=models.CASCADE, related_name='patient_examinations')
    examination = models.ForeignKey('Examination', on_delete=models.CASCADE, null = True, blank = True)
    video = models.OneToOneField('Video', on_delete=models.CASCADE, null = True, blank = True, related_name='patient_examination')
    report_file = models.OneToOneField('ReportFile', on_delete=models.CASCADE, null = True, blank = True, related_name='patient_examination')

    class Meta:
        verbose_name = 'Patient Examination'
        verbose_name_plural = 'Patient Examinations'
        ordering = ['patient', 'examination']

    def __str__(self):
        return f"{self.patient} - {self.report_file}"
    
    def find_matching_video_from_patient(self):
        """
        Finds a video for this patient examination based on the patient's videos.
        For this, the videos date must be the same as the report file's date.
        #TODO add more criteria for matching: Examination type
        """
        videos = self.patient.video_set.filter(date=self.report_file.date, patient_examination__isnull=True)
        if videos:
            if len(videos) > 1:
                print(f"Warning: Found more than one video for patient {self.patient} on date {self.report_file.date}. Choosing the first one.")
            return videos[0]
        else:
            videos = self.patient.video_set.filter(patient_examination__isnull=True)
            if len(videos)==1:
                return videos[0]
            
        return None




from django.db import models

class CaseTemplateRuleValueTypeManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)

class CaseTemplateRuleValueType(models.Model):
    """
    A class representing a case template rule value type.

    Attributes:
        name (str): The name of the case template rule value type.
        distribution_type: One of "single_categorical", "multiple_categorical", "numeric"
        description (str): A description of the case template rule value type.
    """
    DISTRIBUTION_TYPES = [
        ("single_categorical", "single_categorical"),
        ("multiple_categorical", "multiple_categorical"),
        ("numeric", "numeric")
    ]
    objects = CaseTemplateRuleValueTypeManager()

    name = models.CharField(max_length=255)
    name_de = models.CharField(max_length=255, null=True)
    name_en = models.CharField(max_length=255, null=True)
    description = models.TextField(blank=True, null=True)

    distribution_type = models.CharField(max_length=255, choices=DISTRIBUTION_TYPES)


    def natural_key(self):
        return (self.name,)

    def __str__(self):
        return self.name

class CaseTemplateRuleValueManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)
    
class CaseTemplateRuleValue(models.Model):
    """
    A class representing a case template rule value.

    Attributes:
        value (str): The value of the case template rule value.
        case_template_rule (CaseTemplateRule): The case template rule of the case template rule value.
        case_template (CaseTemplate): The case template of the case template rule value.
    """
    objects = CaseTemplateRuleValueManager()

    name = models.CharField(max_length=255)
    description = models.TextField(blank=True, null=True)

    fk_value = models.CharField(max_length=255, null=True, blank=True)
    numeric_value = models.FloatField(null=True, blank=True)
    text_value = models.CharField(max_length=255, null=True, blank=True)

    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name
    
    def get_value(self):
        if self.fk_value:
            return self.fk_value
        elif self.numeric_value:
            return self.numeric_value
        elif self.text_value:
            return self.text_value
        else:
            return None



from django.db import models

class GenderManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)
    
class Gender(models.Model):
    """A class representing gender."""
    objects = GenderManager()

    name = models.CharField(max_length=255)
    name_de = models.CharField(max_length=255, null=True)
    name_en = models.CharField(max_length=255, null=True)
    abbreviation = models.CharField(max_length=255, null=True)
    description = models.TextField(blank=True, null=True)

    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name




from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import Permission
from endoreg_db.models import *  # Import your models here
from django.db import transaction

# Step 1: Define model to category mappings
# Assuming every model class name is unique across your entire Django project
model_categories = {
    'SensitiveModel1': 'sensitive',
    'SensitiveModel2': 'sensitive',
    'DevelopmentModel1': 'development',
    'DevelopmentModel2': 'development',
    # Add all models you have, mapping them to either 'sensitive', 'development', or 'all'
}

# Step 2: Define permissions for each category
category_permissions = {
    'sensitive': ['view', 'edit', 'delete'],
    'development': ['view', 'edit'],
    'all': ['view'],
}

@transaction.atomic
def create_permissions_for_all_models():
    for model_class_name, category in model_categories.items():
        model_class = globals().get(model_class_name)
        if model_class is None:
            print(f"Model {model_class_name} not found.")
            continue
        
        content_type = ContentType.objects.get_for_model(model_class)
        permissions = category_permissions.get(category, [])

        for permission_codename in permissions:
            permission_name = f"Can {permission_codename} {model_class_name}"
            permission, created = Permission.objects.get_or_create(
                codename=f"{permission_codename}_{model_class_name.lower()}",
                defaults={'name': permission_name, 'content_type': content_type},
            )
            if created:
                print(f"Created permission: {permission_name}")

# Run the function to create and assign permissions based on categories
create_permissions_for_all_models()



from .gender import Gender
from .person import Person
from .patient import *
from .examiner import Examiner, ExaminerSerializer
from .portal_user_information import PortalUserInfo, Profession
from .first_name import FirstName
from .last_name import LastName


from django.core.management.base import BaseCommand
from django.apps import apps
from django.db import transaction

class Command(BaseCommand):
    help = 'Deletes all objects from all models in the "endoreg_db" app.'

    def handle(self, *args, **options):
        app_models = apps.get_models()
        app_models = [model for model in app_models if "endoreg_db" in model._meta.app_label]  # Filter models by the "endoreg_db" app
        for model in app_models:
            with transaction.atomic():  # Use a separate transaction for each model
                try:
                    model.objects.all().delete()
                    self.stdout.write(self.style.SUCCESS(f'Successfully deleted all objects in {model._meta.db_table}'))
                except Exception as e:
                    # This catches exceptions and continues with the next model
                    self.stdout.write(self.style.ERROR(f'Error deleting objects in {model._meta.db_table}: {e}'))



from django.db import models

class PatientLabValue(models.Model):
    """
    A class representing a patient lab value.

    Attributes:
        patient (Patient): The patient.
        lab_value (LabValue): The lab value.
        value (float): The value of the lab value.
        date (datetime): The date of the lab value.
    """
    patient = models.ForeignKey('Patient', on_delete=models.CASCADE,
                                related_name="lab_values", blank=True, null=True
    )
    lab_value = models.ForeignKey('LabValue', on_delete=models.CASCADE)
    value = models.FloatField(blank=True, null=True)
    value_str = models.CharField(max_length=255, blank=True, null=True)
    sample = models.ForeignKey(
        'PatientLabSample', on_delete=models.CASCADE, 
        blank=True, null=True,
        related_name='values'
    )
    datetime = models.DateTimeField(# if not set, use now
        auto_now_add=True        
    )
    normal_range = models.JSONField(
        default = dict
    )
    unit = models.ForeignKey('Unit', on_delete=models.CASCADE, blank=True, null=True)

    @classmethod
    def create_lab_value_by_sample(cls, sample=None, lab_value_name=None, value=None, value_str=None, unit=None):
        from endoreg_db.models import LabValue
        patient = sample.patient
        lab_value = LabValue.objects.get(name=lab_value_name)
        value = value
        value_str = value_str

        pat_lab_val = cls.objects.create(
            patient = patient,
            lab_value = lab_value,
            value = value,
            value_str = value_str,
            sample = sample,
            unit = unit,
        )

        pat_lab_val.save()

        return pat_lab_val

    def __str__(self):
        _str = f'{self.lab_value} - {self.value} {self.unit} ({self.datetime})'
        print(_str)
        return _str
    
    def set_min_norm_value(self, value, save = True):
        self.normal_range["min"] = value
        if save:
            self.save()

    def set_max_norm_value(self, value, save = True):
        self.normal_range["max"] = value
        if save:
            self.save()

    def set_norm_values_from_default(self):
        age = self.patient.age()
        gender = self.patient.gender
        min_value, max_value = self.lab_value.get_normal_range(age=age, gender=gender)
        self.set_min_norm_value(min_value, save = False)
        self.set_max_norm_value(max_value, save = False)
        self.save()


    def set_unit_from_default(self):
        self.unit = self.lab_value.default_unit
        self.save()

    def get_value(self):
        if self.value:
            return self.value
        else:
            return self.value_str
        
    def get_value_field_name(self):
        if self.value:
            return "value"
        else:
            return "value_str"
        
    # customize save method so that if a numeric value exists, we round it to the precision of the lab value
    def save(self, *args, **kwargs):
        if self.value:
            precision = self.lab_value.numeric_precision
            self.value = round(self.value, precision)
        super().save(*args, **kwargs)

    def set_value_by_distribution(self, distribution=None, save = True):
        from endoreg_db.models import (
            Patient, LabValue, Gender,
            DateValueDistribution,
            SingleCategoricalValueDistribution,
            NumericValueDistribution,
            MultipleCategoricalValueDistribution,
        ) 
        import warnings  

        patient:Patient = self.patient

        dob = patient.dob
        gender:Gender = patient.gender
        lab_value:LabValue = self.lab_value

        assert self.lab_value, "Lab value must be set to set value by distribution"
        self.unit = self.lab_value.default_unit

        if not distribution:
            distribution = lab_value.get_default_default_distribution()

            if not distribution:
                warnings.warn(
                    "No distribution set for lab value, assuming uniform numeric distribution based on normal values"
                )

                if not self.normal_range.get("min", None) or not self.normal_range.get("max", None):
                    self.set_norm_values_from_default()

                self.normal_range:dict
                _min = self.normal_range.get("min", 0.0001)
                _max = self.normal_range.get("max", 100)
                _name = "auto-" + self.lab_value.name + "-distribution-default-uniform" 
                distribution = NumericValueDistribution(
                    name = _name,
                    min_value = _min,
                    max_value = _max,
                    distribution_type = "uniform"
                )

                value = distribution.generate_value()
                self.value = value
                if save:
                    self.save()
                
                return value

        if isinstance(distribution, SingleCategoricalValueDistribution):
            value = distribution.generate_value()
            self.value_str = value
            if save:
                self.save()
            return value
        
        elif isinstance(distribution, NumericValueDistribution):
            value = distribution.generate_value()
            self.value = value
            if save:
                self.save()
            return value
        
        elif isinstance(distribution, MultipleCategoricalValueDistribution):
            value = distribution.generate_value()
            self.value_str = value
            if save:
                self.save()
            return value

        elif isinstance(distribution, DateValueDistribution):
            # raise not implemented error
            value = distribution.generate_value()
            self.value = value
            if save:
                self.save()





from .frame import Frame, LegacyFrame
from .report_file import ReportFile
from .video import Video, LegacyVideo, VideoImportMeta
from .video_segment import LegacyLabelVideoSegment, LabelVideoSegment
from .metadata import *
from .import_classes import *


from django.db import models

class Event(models.Model):
    """
    A class representing an event.

    Attributes:
        name (str): The name of the event.
        name_de (str): The German name of the event.
        name_en (str): The English name of the event.
        description (str): A description of the event.
    """
    name = models.CharField(max_length=100, unique=True)
    name_de = models.CharField(max_length=100, blank=True, null=True)
    name_en = models.CharField(max_length=100, blank=True, null=True)
    description = models.TextField(blank=True, null=True)

    def natural_key(self):
        return (self.name,)

    def __str__(self):
        return self.name


from ...models import LabelSet, ImageClassificationAnnotation
from django.db.models import Q, F
from django.db import models
from icecream import ic
from tqdm import tqdm
from collections import defaultdict

# def get_legacy_annotations_for_labelset(labelset_name, version=None):
#     """
#     Retrieve annotations for a given label set for training.

#     Args:
#     - labelset_name (str): The name of the label set.
#     - version (int, optional): The version of the label set. If not specified, the latest version is fetched.

#     Returns:
#     - list[dict]: A list of dictionaries. Each dictionary represents an image and its annotations.
#                   Format: [{"frame": <frame_object>, "annotations": [{"label": <label_name>, "value": <value>}, ...]}, ...]

#     Example:
#         annotations_for_training = get_annotations_for_labelset("YourLabelSetName", version=2)

#     """

#     # Fetch the label set based on the name and optionally the version
#     if version:
#         labelset = LabelSet.objects.get(name=labelset_name, version=version)
#     else:
#         labelset = LabelSet.objects.filter(name=labelset_name).order_by('-version').first()
#         if not labelset:
#             raise ValueError(f"No label set found with the name: {labelset_name}")

#     # Retrieve all labels in the label set
#     labels_in_set = labelset.labels.all()

#     # Get the most recent annotations for each frame/label combination
#     annotations = ImageClassificationAnnotation.objects.filter(label__in=labels_in_set)
#     annotations = annotations.annotate(
#         latest_annotation=models.Window(
#             expression=models.functions.RowNumber(),
#             partition_by=[F('legacy_image'), F('label')],
#             order_by=F('date_modified').desc()
#         )
#     ).filter(latest_annotation=1)

#     # Organize the annotations by image/frame
#     organized_annotations = []

#     for annotation in tqdm(annotations):
#         # ic(annotation)
#         # Check if the frame is already in the organized list
#         existing_entry = next((entry for entry in organized_annotations if entry['legacy_image'] == annotation.legacy_frame), None)

#         if existing_entry:
#             # Add this annotation to the existing frame's annotations
#             existing_entry['annotations'].append({
#                 "label": annotation.label.name,
#                 "value": annotation.value
#             })
#         else:
#             # Create a new entry for this frame
#             organized_annotations.append({
#                 "legacy_image": annotation.legacy_image,
#                 "annotations": [{
#                     "label": annotation.label.name,
#                     "value": annotation.value
#                 }]
#             })

#     return organized_annotations



def get_legacy_annotations_for_labelset(labelset_name, version=None):
    """
    ... [rest of your docstring]
    """

    # Fetch the label set based on the name and optionally the version
    if version:
        labelset = LabelSet.objects.get(name=labelset_name, version=version)
    else:
        labelset = LabelSet.objects.filter(name=labelset_name).order_by('-version').first()
        if not labelset:
            raise ValueError(f"No label set found with the name: {labelset_name}")

    # Retrieve all labels in the label set
    labels_in_set = labelset.labels.all()

    # Get the most recent annotations for each frame/label combination
    annotations = (ImageClassificationAnnotation.objects
                   .filter(label__in=labels_in_set)
                   .select_related('legacy_image', 'label')  # Reduce number of queries
                   .annotate(
                        latest_annotation=models.Window(
                            expression=models.functions.RowNumber(),
                            partition_by=[F('legacy_image'), F('label')],
                            order_by=F('date_modified').desc()
                        )
                    ).filter(latest_annotation=1))

    # Organize the annotations by image/frame using a defaultdict
    organized_annotations_dict = defaultdict(lambda: {
        "legacy_image": None,
        "annotations": []
    })

    for annotation in tqdm(annotations):
        organized_entry = organized_annotations_dict[annotation.legacy_image.id]
        organized_entry["legacy_image"] = annotation.legacy_image
        organized_entry["annotations"].append({
            "label": annotation.label.name,
            "value": annotation.value
        })

    # Convert organized_annotations_dict to a list
    organized_annotations = list(organized_annotations_dict.values())

    return organized_annotations

def generate_legacy_dataset_output(labelset_name, version=None):
    """
    Generate an output suitable for creating PyTorch datasets.

    Args:
    - labelset_name (str): The name of the label set.
    - version (int, optional): The version of the label set. If not specified, the latest version is fetched.

    Returns:
    - list[dict]: A list of dictionaries, where each dictionary contains the file path and the labels.
                  Format: [{"path": <file_path>, "labels": [<label_1_value>, <label_2_value>, ...]}, ...]
    - labelset[LabelSet]: The label set that was used to generate the output.
    """

    # First, retrieve the organized annotations using the previously defined function
    organized_annotations = get_legacy_annotations_for_labelset(labelset_name, version)

    # Fetch all labels from the labelset for consistent ordering
    labelset = LabelSet.objects.get(name=labelset_name, version=version)
    all_labels = labelset.get_labels_in_order()

    dataset_output = []

    for entry in organized_annotations:
        # Prepare a dictionary for each frame
        frame_data = {
            "path": entry['legacy_image'].image.path,  # Assuming 'image' field stores the file path
            "labels": [-1] * len(all_labels)  # Initialize with -1 for all labels
        }

        # Update the labels based on the annotations
        for annotation in entry['annotations']:
            index = next((i for i, label in enumerate(all_labels) if label.name == annotation['label']), None)
            if index is not None:
                frame_data['labels'][index] = int(annotation['value'])

        dataset_output.append(frame_data)

    return dataset_output, labelset



from django.db import models

class SensitiveMeta(models.Model):
    examination_date = models.DateField(blank=True, null=True)
    patient_first_name = models.CharField(max_length=255, blank=True, null=True)
    patient_last_name = models.CharField(max_length=255, blank=True, null=True)
    patient_dob = models.DateField(blank=True, null=True)
    endoscope_type = models.CharField(max_length=255, blank=True, null=True)
    endoscope_sn = models.CharField(max_length=255, blank=True, null=True)

    @classmethod
    def create_from_dict(cls, data: dict):
        # data can contain more fields than the model has
        field_names = [_.name for _ in cls._meta.fields]
        selected_data = {k: v for k, v in data.items() if k in field_names}

        return cls.objects.create(**selected_data)
    
    def update_from_dict(self, data: dict):
        # data can contain more fields than the model has
        field_names = [_.name for _ in self._meta.fields]
        selected_data = {k: v for k, v in data.items() if k in field_names}

        for k, v in selected_data.items():
            setattr(self, k, v)
        
        self.save()
    
    def __str__(self):
        return f"SensitiveMeta: {self.examination_date} {self.patient_first_name} {self.patient_last_name} (*{self.patient_dob})"
    



# import cv2
from pathlib import Path
import datetime

def _extract_metadata_from_filename(filepath:Path, time_format = None): # deprecated
    """
    Extracts metadata from a video filename.

    Args:
        filepath (Path): The path to the video file.
        time_format (str, optional): The format of the date in the filename. Defaults to None.

    Returns:
        dict: A dictionary containing the extracted metadata.

    Example return value:
        {
            'examination_date': '2021-03-03',
            'patient_last_name': 'Doe',
            'patient_first_name': 'John',
            'patient_dob': '1990-01-01'
        }
    """
    # get filename without suffix
    filename = filepath.stem    

    if not time_format:
        time_format = "%d.%m.%Y"

    _info = [_.strip() for _ in filename.split("_")]

    if len(_info) == 4:
        examination_date, last_name, first_name, birthdate = _info
    
    else:
        examination_date, last_name, first_name, birthdate = None, None, None, None

    metadata = {}
    metadata['examination_date'] = examination_date
    metadata['patient_last_name'] = last_name
    metadata['patient_first_name'] = first_name
    metadata['patient_dob'] = birthdate

    try:
        metadata['examination_date'] = datetime.datetime.strptime(examination_date, time_format).date()
        metadata['examination_date'] = metadata['examination_date'].strftime("%Y-%m-%d")
    except:
        metadata['examination_date'] = None

    try:
        metadata['patient_dob'] = datetime.datetime.strptime(birthdate, time_format).date()
        metadata['patient_dob'] = metadata['patient_dob'].strftime("%Y-%m-%d")
    except:
        metadata['patient_dob'] = None

    return metadata

def _get_video_metadata(file_path): # Deprecated
    """
    Returns the framerate, dimensions, and duration of a video file.

    Parameters:
    file_path (str): The path to the video file.

    Returns:
    tuple: A tuple containing the framerate (float), dimensions (tuple of ints), and duration (float) of the video.
    """
    # Open the video file
    video = cv2.VideoCapture(file_path)

    # Get the video framerate
    fps = video.get(cv2.CAP_PROP_FPS)

    # Get the video dimensions
    width = int(video.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(video.get(cv2.CAP_PROP_FRAME_HEIGHT))
    dimensions = (width, height)

    # Get the video duration
    frame_count = int(video.get(cv2.CAP_PROP_FRAME_COUNT))
    duration = frame_count / fps

    # Release the video file
    video.release()

    # Return the metadata
    return fps, dimensions, duration



from .product import Product
from .product_material import ProductMaterial
from .product_group import ProductGroup
from .reference_product import ReferenceProduct
from .product_weight import ProductWeight


from rest_framework import serializers
from endoreg_db.models import Frame, LegacyFrame

class FrameSerializer(serializers.ModelSerializer):
    class Meta:
        model = Frame
        fields = '__all__'

class LegacyFrameSerializer(serializers.ModelSerializer):
    class Meta:
        model = LegacyFrame
        fields = '__all__'




# import cv2
from PIL import Image
from django.core.files.base import ContentFile
from django.db import models, transaction
from tqdm import tqdm
# import cv2
import io
from datetime import date

BATCH_SIZE = 1000

class AbstractVideo(models.Model):
    file = models.FileField(upload_to="raw_videos", blank=True, null=True)
    video_hash = models.CharField(max_length=255, unique=True)
    patient = models.ForeignKey("Patient", on_delete=models.CASCADE, blank=True, null=True)
    date = models.DateField(blank=True, null=True)
    suffix = models.CharField(max_length=255)
    fps = models.FloatField()
    duration = models.FloatField()
    width = models.IntegerField()
    height = models.IntegerField()
    endoscope_image_x = models.IntegerField(blank=True, null=True)
    endoscope_image_y = models.IntegerField(blank=True, null=True)
    endoscope_image_width = models.IntegerField(blank=True, null=True)
    endoscope_image_height = models.IntegerField(blank=True, null=True)
    center = models.ForeignKey("Center", on_delete=models.CASCADE, blank=True, null=True)
    endoscopy_processor = models.ForeignKey("EndoscopyProcessor", on_delete=models.CASCADE, blank=True, null=True)
    frames_extracted = models.BooleanField(default=False)

    meta = models.JSONField(blank=True, null=True)

    class Meta:
        abstract = True

    def get_roi_endoscope_image(self):
        return {
            'x': self.endoscope_image_content_x,
            'y': self.endoscope_image_content_y,
            'width': self.endoscope_image_content_width,
            'height': self.endoscope_image_content_height,
        }

    def initialize_metadata_in_db(self, video_meta=None):
        if not video_meta:
            video_meta = self.meta
        self.set_examination_date_from_video_meta(video_meta)
        self.patient, created = self.get_or_create_patient(video_meta)
        self.save()

    def get_or_create_patient(self, video_meta=None):
        from ...persons import Patient
        if not video_meta:
            video_meta = self.meta

        patient_first_name = video_meta['patient_first_name']
        patient_last_name = video_meta['patient_last_name']
        patient_dob = video_meta['patient_dob']

        # assert that we got all the necessary information
        assert patient_first_name and patient_last_name and patient_dob, "Missing patient information"

        patient, created = Patient.objects.get_or_create(
            first_name=patient_first_name,
            last_name=patient_last_name,
            dob=patient_dob
        )

        return patient, created

    def get_frame_model(self):
        assert 1 == 2, "This method should be overridden in derived classes"

    def get_video_model(self):
        assert 1 == 2, "This method should be overridden in derived classes"

    def get_frame_number(self):
        """
        Get the number of frames in the video.
        """
        frame_model = self.get_frame_model()
        framecount = frame_model.objects.filter(video=self).count()
        return framecount
    
    def set_frames_extracted(self, value:bool=True):
        self.frames_extracted = value
        self.save()
        
    def get_frames(self):
        """
        Retrieve all frames for this video in the correct order.
        """
        frame_model = self.get_frame_model()
        return frame_model.objects.filter(video=self).order_by('frame_number')

    def get_frame(self, frame_number):
        """
        Retrieve a specific frame for this video.
        """
        frame_model = self.get_frame_model()
        return frame_model.objects.get(video=self, frame_number=frame_number)

    def get_frame_range(self, start_frame_number:int, end_frame_number:int):
        """
        Expects numbers of start and stop frame.
        Returns all frames of this video within the given range in ascending order.
        """
        frame_model = self.get_frame_model()
        return frame_model.objects.filter(video=self, frame_number__gte=start_frame_number, frame_number__lte=end_frame_number).order_by('frame_number')

    def _create_frame_object(self, frame_number, image_file):
        frame_model = self.get_frame_model()
        frame = frame_model(
                video=self,
                frame_number=frame_number,
                suffix='jpg',
            )
        frame.image_file = image_file  # Temporary store the file-like object

        return frame

    def _bulk_create_frames(self, frames_to_create):
        frame_model = self.get_frame_model()
        with transaction.atomic():
            frame_model.objects.bulk_create(frames_to_create)

            # After the DB operation, save the ImageField for each object
            for frame in frames_to_create:
                frame_name = f"video_{self.id}_frame_{str(frame.frame_number).zfill(7)}.jpg"
                frame.image.save(frame_name, frame.image_file)

            # Clear the list for the next batch
            frames_to_create = []

    def set_examination_date_from_video_meta(self, video_meta=None):
        if not video_meta:
            video_meta = self.meta
        date_str = video_meta['examination_date'] # e.g. 2020-01-01
        if date_str:
            self.date = date.fromisoformat(date_str)
            self.save()

    def extract_all_frames(self):
        """
        Extract all frames from the video and store them in the database.
        Uses Django's bulk_create for more efficient database operations.
        """
        # Open the video file
        video = cv2.VideoCapture(self.file.path)

        # Initialize video properties
        self.initialize_video_specs(video)

        # Prepare for batch operation
        frames_to_create = []

        # Extract frames
        for frame_number in tqdm(range(int(self.duration * self.fps))):
            # Read the frame
            success, image = video.read()
            if not success:
                break

            # Convert the numpy array to a PIL Image object
            pil_image = Image.fromarray(cv2.cvtColor(image, cv2.COLOR_BGR2RGB))

            # Save the PIL Image to a buffer
            buffer = io.BytesIO()
            pil_image.save(buffer, format='JPEG')

            # Create a file-like object from the byte data in the buffer
            image_file = ContentFile(buffer.getvalue())

            # Prepare Frame instance (don't save yet)
            frame = self._create_frame_object(frame_number, image_file)
            frames_to_create.append(frame)

            # Perform bulk create when reaching BATCH_SIZE
            if len(frames_to_create) >= BATCH_SIZE:
                self._bulk_create_frames(frames_to_create)
                frames_to_create = []


        # Handle remaining frames
        if frames_to_create:
            self._bulk_create_frames(frames_to_create)
            frames_to_create = []

        # Close the video file
        video.release()
        self.set_frames_extracted(True)


    def initialize_video_specs(self, video):
        """
        Initialize and save video metadata like framerate, dimensions, and duration.
        """
        self.fps = video.get(cv2.CAP_PROP_FPS)
        self.width = int(video.get(cv2.CAP_PROP_FRAME_WIDTH))
        self.height = int(video.get(cv2.CAP_PROP_FRAME_HEIGHT))
        self.duration = video.get(cv2.CAP_PROP_FRAME_COUNT) / self.fps
        self.save()


from django.db import models

class MedicationManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)

class Medication(models.Model):
    name = models.CharField(max_length=255, unique=True)
    name_de = models.CharField(max_length=255, blank=True, null=True)
    name_en = models.CharField(max_length=255, blank=True, null=True)
    adapt_to_renal_function = models.BooleanField(default = False)
    adapt_to_hepatic_function = models.BooleanField(default=False)
    adapt_to_indication = models.BooleanField(default=False)
    adapt_to_age = models.BooleanField(default=False)
    adapt_to_weight = models.BooleanField(default=False)
    adapt_to_risk = models.BooleanField(default=False)
    default_unit = models.ForeignKey('Unit', on_delete=models.CASCADE)


    objects = MedicationManager()

    def natural_key(self):
        return (self.name,)

    def __str__(self):
        return self.name
    
class MedicationScheduleManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)
    
class MedicationSchedule(models.Model):
    name = models.CharField(max_length=255)
    name_de = models.CharField(max_length=255, blank=True, null=True)
    name_en = models.CharField(max_length=255, blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    medication = models.ForeignKey("Medication", on_delete=models.CASCADE)
    unit = models.ForeignKey("Unit", on_delete=models.CASCADE)
    therapy_duration_d = models.FloatField(blank=True, null=True)
    dose = models.FloatField()
    intake_times = models.ManyToManyField(
        "MedicationIntakeTime", 
    )

    objects = MedicationScheduleManager()

    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name
    
    
class MedicationIntakeTimeManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)
    
class MedicationIntakeTime(models.Model):
    name = models.CharField(max_length=255)
    name_de = models.CharField(max_length=255, blank=True, null=True)
    name_en = models.CharField(max_length=255, blank=True, null=True)
    repeats = models.CharField(max_length=20, default = "daily")
    time = models.TimeField()

    objects = MedicationIntakeTimeManager()

    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name

# IMPLEMENT MEDICATION INDICATION TYPE
class MedicationIndicationTypeManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)

class MedicationIndicationType(models.Model):
    name = models.CharField(max_length=255, unique=True)
    name_de = models.CharField(max_length=255, blank=True, null=True)
    name_en = models.CharField(max_length=255, blank=True, null=True)

    objects = MedicationIndicationTypeManager()

    def natural_key(self):
        return (self.name,)

    def __str__(self):
        return self.name
    
    @classmethod
    def get_random_indication_by_type(cls, name) -> "MedicationIndication":
        return cls.objects.get(name=name).medication_indications.order_by('?').first()
    

    def get_random_medication_indication(self):
        from endoreg_db.models import MedicationIndication
        return MedicationIndication.objects.filter(indication_type=self).order_by('?').first()


class MedicationIndicationManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)

class MedicationIndication(models.Model):
    name = models.CharField(max_length=255, unique=True)
    indication_type = models.ForeignKey(
        "MedicationIndicationType", on_delete=models.CASCADE, related_name="medication_indications"
    )
    medication_schedules = models.ManyToManyField(
        "MedicationSchedule"
    )
    diseases = models.ManyToManyField(
        "Disease"
    )
    events = models.ManyToManyField(
        "Event"
    )
    classification_choices = models.ManyToManyField(
        "DiseaseClassificationChoice"
    )
    sources = models.ManyToManyField(
        "InformationSource"
    )

    def get_indication_links(self):
        links = {
            "medication_schedules": self.medication_schedules,
            "diseases": self.diseases,
            "events": self.events,
            "classification_choices": self.classification_choices
        }

    objects = MedicationIndicationManager()

    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name
    
    def create_patient_medication_schedules(self, patient):
        from endoreg_db.models import PatientMedicationSchedule
        for medication_schedule in self.medication_schedules.all():
            PatientMedicationSchedule.objects.create(
                patient=patient,
                medication_schedule=medication_schedule
            )



from os import name
from django.db import models

class CenterResource(models.Model):
    name = models.CharField(max_length=255, null=True)
    center = models.ForeignKey("Center", on_delete=models.CASCADE)
    name_de = models.CharField(max_length=255, null=True)
    name_en = models.CharField(max_length=255, null=True)
    quantity = models.FloatField()
    resource = models.ForeignKey("Resource", on_delete=models.CASCADE)
    transport_emission_factor = models.ForeignKey("EmissionFactor", on_delete=models.SET_NULL, null=True,
                                                  related_name="center_resource_transport_emission_factor")
    use_emission_factor = models.ForeignKey("EmissionFactor", on_delete=models.SET_NULL, null=True,
                                            related_name="center_resource_use_emission_factor")
    year = models.IntegerField()
    unit = models.ForeignKey("Unit", on_delete=models.SET_NULL, null=True)
    
    def __str__(self):
        return self.center.name + " - " + self.resource.name + " (" + str(self.year) + ")"


from django.db import models

class TransportRouteManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)
    
class TransportRoute(models.Model):
    objects = TransportRouteManager()

    distance = models.FloatField()
    name = models.CharField(max_length=255)
    name_de = models.CharField(max_length=255, null=True)
    name_en = models.CharField(max_length=255, null=True)
    emission_factor = models.ForeignKey("EmissionFactor", on_delete=models.SET_NULL, null=True)
    unit = models.ForeignKey("Unit", on_delete=models.SET_NULL, null=True)

    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name


from django.db import models
import ffmpeg
from pathlib import Path

# import endoreg_center_id from django settings
from django.conf import settings

# check if endoreg_center_id is set
if not hasattr(settings, 'ENDOREG_CENTER_ID'):
    ENDOREG_CENTER_ID = 9999
else:
    ENDOREG_CENTER_ID = settings.ENDOREG_CENTER_ID

# VideoMeta
class VideoMeta(models.Model):
    processor = models.ForeignKey('EndoscopyProcessor', on_delete=models.CASCADE, blank=True, null=True)
    endoscope = models.ForeignKey('Endoscope', on_delete=models.CASCADE, blank=True, null=True)
    center = models.ForeignKey('Center', on_delete=models.CASCADE)
    import_meta = models.OneToOneField('VideoImportMeta', on_delete=models.CASCADE, blank=True, null=True)
    ffmpeg_meta = models.OneToOneField('FFMpegMeta', on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):

        processor_name = self.processor.name if self.processor is not None else "None"
        endoscope_name = self.endoscope.name if self.endoscope is not None else "None"
        center_name = self.center.name if self.center is not None else "None"

        result_html = ""

        result_html += f"Processor: {processor_name}<br>"
        result_html += f"Endoscope: {endoscope_name}<br>"
        result_html += f"Center: {center_name}<br>"

        return result_html
    
    # import meta should be created when video meta is created
    def save(self, *args, **kwargs):
        if self.import_meta is None:
            self.import_meta = VideoImportMeta.objects.create()
        super(VideoMeta, self).save(*args, **kwargs)

    def initialize_ffmpeg_meta(self, file_path):
        """Initializes FFMpeg metadata for the video file if not already done."""
        self.ffmpeg_meta = FFMpegMeta.create_from_file(Path(file_path))
        self.save()

    def update_meta(self, file_path):
        """Updates the video metadata from the file."""
        self.initialize_ffmpeg_meta(file_path)
        self.save()

    def get_endo_roi(self):
        endo_roi = self.processor.get_roi_endoscope_image()
        return endo_roi

    def get_fps(self):
        if not self.ffmpeg_meta:
            return None
    
        return self.ffmpeg_meta.frame_rate
 

class FFMpegMeta(models.Model):
    # Existing fields
    duration = models.FloatField(blank=True, null=True)
    width = models.IntegerField(blank=True, null=True)
    height = models.IntegerField(blank=True, null=True)
    frame_rate = models.FloatField(blank=True, null=True)
    
    # New fields for comprehensive information
    video_codec = models.CharField(max_length=50, blank=True, null=True)
    audio_codec = models.CharField(max_length=50, blank=True, null=True)
    audio_channels = models.IntegerField(blank=True, null=True)
    audio_sample_rate = models.IntegerField(blank=True, null=True)

    # Existing __str__ method can be updated to include new fields

    @classmethod
    def create_from_file(cls, file_path: Path):
        """Creates an FFMpegMeta instance from a video file using ffmpeg probe."""
        try:
            probe = ffmpeg.probe(file_path.resolve().as_posix())
        except: # ffmpeg.Error as e:
            # print(e.stderr)
            print(f"Error while probing {file_path}")
            return None

        video_stream = next((stream for stream in probe['streams'] if stream['codec_type'] == 'video'), None)
        audio_streams = [stream for stream in probe['streams'] if stream['codec_type'] == 'audio']

        # Check for the existence of a video stream
        if video_stream is None:
            print(f"No video stream found in {file_path}")
            return None

        # Extract and store video metadata
        metadata = {
            'duration': float(video_stream.get('duration', 0)),
            'width': int(video_stream.get('width', 0)),
            'height': int(video_stream.get('height', 0)),
            'frame_rate': float(next(iter(video_stream.get('avg_frame_rate', '').split('/')), 0)),
            'video_codec': video_stream.get('codec_name', ''),
        }

        # If there are audio streams, extract and store audio metadata from the first stream
        if audio_streams:
            first_audio_stream = audio_streams[0]
            metadata.update({
                'audio_codec': first_audio_stream.get('codec_name', ''),
                'audio_channels': int(first_audio_stream.get('channels', 0)),
                'audio_sample_rate': int(first_audio_stream.get('sample_rate', 0)),
            })

        # Create and return the FFMpegMeta instance
        return cls.objects.create(**metadata)

class VideoImportMeta(models.Model):

    video_anonymized = models.BooleanField(default=False)
    video_patient_data_detected = models.BooleanField(default=False)
    outside_detected = models.BooleanField(default=False)
    patient_data_removed = models.BooleanField(default=False)
    outside_removed = models.BooleanField(default=False)

    def __str__(self):
        result_html = ""

        result_html += f"Video anonymized: {self.video_anonymized}<br>"
        result_html += f"Video patient data detected: {self.video_patient_data_detected}<br>"
        result_html += f"Outside detected: {self.outside_detected}<br>"
        result_html += f"Patient data removed: {self.patient_data_removed}<br>"
        result_html += f"Outside removed: {self.outside_removed}<br>"
        return result_html


import os
from pathlib import Path

from .raw_video import RawVideoFile
from .raw_pdf import RawPdfFile

# FileImporter class
# This class is used to import data from a file into the database.
# Expects a directory containing files to import.
# creates correct import file object depending on file type

# FileImporter class
# This class is used to import data from a file into the database by creating objects for the files.
# main method is import_files which expects a path to a directory containing files to import.
# creates correct import file object depending on file type by checking the file extension

class FileImporter:
    def __init__(self, directory):
        self.directory = directory

    def import_files(self):
        directory_path = Path(self.directory)
        for file in directory_path.iterdir():
            if file.is_file():
                if file.suffix.lower() in ['.mov', '.mp4']:
                    RawVideoFile.create_from_file(file)
                else:
                    raise ValueError(f"File type {file.suffix} not supported")
            else:
                raise ValueError(f"{file} is not a file")
            
        


from django.db import models
from pathlib import Path
from collections import defaultdict, Counter

from endoreg_db.utils.hashs import get_video_hash
from endoreg_db.utils.file_operations import get_uuid_filename
from endoreg_db.utils.ocr import extract_text_from_rois

import shutil
import os
import subprocess

from ..metadata import VideoMeta, SensitiveMeta

class RawVideoFile(models.Model):
    uuid = models.UUIDField()
    file = models.FileField(upload_to="raw_data/")
    
    sensitive_meta = models.OneToOneField(
        "SensitiveMeta", on_delete=models.CASCADE, blank=True, null=True
    ) 

    center = models.ForeignKey("Center", on_delete=models.CASCADE)
    processor = models.ForeignKey(
        "EndoscopyProcessor", on_delete=models.CASCADE, blank=True, null=True
    )
    video_meta = models.OneToOneField(
        "VideoMeta", on_delete=models.CASCADE, blank=True, null=True
    )
    original_file_name = models.CharField(max_length=255)
    video_hash = models.CharField(max_length=255, unique=True)
    uploaded_at = models.DateTimeField(auto_now_add=True)

    # Frame Extraction States
    state_frames_required = models.BooleanField(default=True)
    state_frames_extracted = models.BooleanField(default=False)

    # Video
    ## Prediction
    state_initial_prediction_required = models.BooleanField(default=True)
    state_initial_prediction_completed = models.BooleanField(default=False)
    state_initial_prediction_import_required = models.BooleanField(default=True)
    state_initial_prediction_import_completed = models.BooleanField(default=False)
    ## OCR
    state_ocr_required = models.BooleanField(default=True)
    state_ocr_completed = models.BooleanField(default=False)
    ## Validation
    state_outside_validated = models.BooleanField(default=False)
    state_ocr_result_validated = models.BooleanField(default=False)

    state_sensitive_data_retrieved = models.BooleanField(default=False)

    # Dataset complete?
    state_histology_required = models.BooleanField(blank=True, null=True)
    state_histology_available = models.BooleanField(default=False)
    state_follow_up_intervention_required = models.BooleanField(blank=True, null=True)
    state_follow_up_intervention_available = models.BooleanField(default=False)
    state_dataset_complete = models.BooleanField(default=False)

    # Finalizing for Upload
    state_anonym_video_required = models.BooleanField(default=True)
    state_anonym_video_performed = models.BooleanField(default=False)
    state_original_reports_deleted = models.BooleanField(default=False)
    state_original_video_deleted = models.BooleanField(default=False)
    state_finalized = models.BooleanField(default=False)

    frame_dir = models.CharField(max_length=255)
    prediction_dir = models.CharField(max_length=255)

    @classmethod
    def create_from_file(
        cls,
        file_path: Path,
        video_dir: Path,
        center_name: str,
        processor_name: str,
        frame_dir_parent: Path,
        save: bool = True,
    ):
        from endoreg_db.models import Center, EndoscopyProcessor

        print(f"Creating RawVideoFile from {file_path}")
        original_file_name = file_path.name
        # Rename and and move

        new_file_name, uuid = get_uuid_filename(file_path)
        framedir: Path = frame_dir_parent / str(uuid)

        if not framedir.exists():
            framedir.mkdir(parents=True, exist_ok=True)

        if not video_dir.exists():
            video_dir.mkdir(parents=True, exist_ok=True)

        video_hash = get_video_hash(file_path)

        center = Center.objects.get(name=center_name)
        assert center is not None, "Center must exist"

        processor = EndoscopyProcessor.objects.get(name=processor_name)
        assert processor is not None, "Processor must exist"

        new_filepath = video_dir / new_file_name

        print(f"Moving {file_path} to {new_filepath}")
        shutil.move(file_path.resolve().as_posix(), new_filepath.resolve().as_posix())
        print(f"Moved to {new_filepath}")

        # Make sure file was transferred correctly and hash is correct
        if not new_filepath.exists():
            print(f"File {file_path} was not transferred correctly to {new_filepath}")
            return None

        new_hash = get_video_hash(new_filepath)
        if new_hash != video_hash:
            print(f"Hash of file {file_path} is not correct")
            return None

        # make sure that no other file with the same hash exists
        if cls.objects.filter(video_hash=video_hash).exists():
            # log and print warnint
            print(f"File with hash {video_hash} already exists")
            return None

        else:
            print(center)
            # Create a new instance of RawVideoFile
            raw_video_file = cls(
                uuid=uuid,
                file=new_filepath.resolve().as_posix(),
                center=center,
                processor=processor,
                original_file_name=original_file_name,
                video_hash=video_hash,
                frame_dir=framedir.as_posix(),
            )

            # Save the instance to the database
            raw_video_file.save()

            return raw_video_file

    def __str__(self):
        return self.file.name

    def get_endo_roi(self):
        endo_roi = self.video_meta.get_endo_roi()
        return endo_roi

    # video meta should be created when video file is created
    def save(self, *args, **kwargs):
        if self.video_meta is None:
            center = self.center
            processor = self.processor
            self.video_meta = VideoMeta.objects.create(
                center=center, processor=processor
            )
            self.video_meta.initialize_ffmpeg_meta(self.file.path)
        super(RawVideoFile, self).save(*args, **kwargs)

    def extract_frames(
        self,
        quality: int = 2,
        frame_dir: Path = None,
        overwrite: bool = False,
        ext="jpg",
    ):
        """
        Extract frames from the video file and save them to the frame_dir.
        For this, ffmpeg must be available in in the current environment.
        """
        if frame_dir is None:
            frame_dir = Path(self.frame_dir)
        else:
            frame_dir = Path(frame_dir)

        if not frame_dir.exists():
            frame_dir.mkdir(parents=True, exist_ok=True)

        if not overwrite and len(list(frame_dir.glob("*.jpg"))) > 0:
            print(f"Frames already extracted for {self.file.name}")
            return

        video_path = Path(self.file.path).resolve().as_posix()

        frame_path_string = frame_dir.resolve().as_posix()
        command = [
            "ffmpeg",
            "-i",
            video_path,  #
            "-q:v",
            str(quality),
            os.path.join(frame_path_string, f"frame_%07d.{ext}"),
        ]

        # Ensure FFmpeg is available
        if not shutil.which("ffmpeg"):
            raise EnvironmentError(
                "FFmpeg could not be found. Ensure it is installed and in your PATH."
            )

        # Extract frames from the video file
        # Execute the command
        result = subprocess.run(command, capture_output=True, text=True)
        if result.returncode != 0:
            raise Exception(f"Error extracting frames: {result.stderr}")

        self.state_frames_extracted = True

        return f"Frames extracted to {frame_dir} ({frame_path_string}) with quality {quality}"

    def delete_frames(self):
        """
        Delete frames extracted from the video file.
        """
        frame_dir = Path(self.frame_dir)
        if frame_dir.exists():
            shutil.rmtree(frame_dir)
            self.state_frames_extracted = False
            self.save()
            return f"Frames deleted from {frame_dir}"
        else:
            return f"No frames to delete for {self.file.name}"

    def get_frame_path(self, n: int = 0):
        """
        Get the path to the n-th frame extracted from the video file.
        Note that the frame numbering starts at 1 in our naming convention.
        """
        # Adjust index
        n = n + 1

        frame_dir = Path(self.frame_dir)
        return frame_dir / f"frame_{n:07d}.jpg"
    
    def get_frame_paths(self):
        if not self.state_frames_extracted:
            return None
        frame_dir = Path(self.frame_dir)
        paths = [p for p in frame_dir.glob('*')]
        indices = [int(p.stem.split("_")[1]) for p in paths]
        path_index_tuples = list(zip(paths, indices))
        # sort ascending by index
        path_index_tuples.sort(key=lambda x: x[1])
        paths, indices = zip(*path_index_tuples)

        return paths

    def get_prediction_dir(self):
        return Path(self.prediction_dir)

    def get_predictions_path(self, suffix = ".json"):
        pred_dir = self.get_prediction_dir()
        return pred_dir.joinpath("predictions").with_suffix(suffix)
    
    def get_smooth_predictions_path(self, suffix = ".json"):
        pred_dir = self.get_prediction_dir()
        return pred_dir.joinpath("smooth_predictions").with_suffix(suffix)
    
    def get_binary_predictions_path(self, suffix = ".json"):
        pred_dir = self.get_prediction_dir()
        return pred_dir.joinpath("binary_predictions").with_suffix(suffix)
    
    def get_raw_sequences_path(self, suffix = ".json"):
        pred_dir = self.get_prediction_dir()
        return pred_dir.joinpath("raw_sequences").with_suffix(suffix)

    def get_filtered_sequences_path(self, suffix=".json"):
        pred_dir = self.get_prediction_dir()
        return pred_dir.joinpath("filtered_sequences").with_suffix(suffix)

    def extract_text_information(self, frame_fraction: float = 0.001):
        """
        Extract text information from the video file.
        Makes sure that frames are extracted and then processes the frames.
        gets all frames from frame_dir and selects a fraction of them to process (at least 1)
        """
        if not self.state_frames_extracted:
            print(f"Frames not extracted for {self.file.name}")
            return None

        processor = self.processor

        frame_dir = Path(self.frame_dir)
        frames = list(frame_dir.glob("*"))
        n_frames = len(frames)
        n_frames_to_process = max(1, int(frame_fraction * n_frames))

        # Select evenly spaced frames
        frames = frames[:: n_frames // n_frames_to_process]

        # extract text from each frame and store the value to
        # defaultdict of lists.
        # Then, extract the most frequent value from each list
        # Finally, return the dictionary of most frequent values

        # Create a defaultdict to store the extracted text from each ROI
        rois_texts = defaultdict(list)

        print(f"Processing {n_frames_to_process} frames from {self.file.name}")
        # Process frames
        for frame_path in frames[:n_frames_to_process]:
            extracted_texts = extract_text_from_rois(frame_path, processor)
            for roi, text in extracted_texts.items():
                rois_texts[roi].append(text)

        # Get the most frequent text values for each ROI using Counter
        for key in rois_texts.keys():
            counter = Counter([text for text in rois_texts[key] if text])
            rois_texts[key] = counter.most_common(1)[0][0] if counter else None

        return rois_texts

    def update_text_metadata(self, ocr_frame_fraction=0.001):
        print(f"Updating metadata for {self.file.name}")
        texts = self.extract_text_information(ocr_frame_fraction)

        self.sensitive_meta = SensitiveMeta.create_from_dict(texts)
        self.state_sensitive_data_retrieved = True
        self.save()

        # Resulting dict depends on defined ROIs for this processor type!

    def update_video_meta(self):
        video_meta = self.video_meta
        video_path = Path(self.file.path)

        if video_meta is None:
            video_meta = VideoMeta.create_from_video(video_path)
            self.video_meta = video_meta
            self.save()

        else:
            video_meta.update_meta(video_path)

    def get_fps(self):
        if self.video_meta is None:
            self.update_video_meta()

        if self.video_meta.ffmpeg_meta is None:
            self.video_meta.initialize_ffmpeg_meta(self.file.path)

        return self.video_meta.get_fps()



from ..raw_pdf import RawPdfFile
from logging import getLogger
#
# # setup logging to pdf_import.log
logger = getLogger('examination_pdf_import')

def get_pdf_files_scheduled_for_processing():
    reports = RawPdfFile.objects.filter(
        state_report_processing_required=True
    )
    return reports

def process_pdf_file(report:RawPdfFile):
    if report.update(save=True, verbose=True):
        logger.info(f"Report {report} processed successfully")
        return True
    else:
        logger.error(f"Report {report} processing failed")
        return False


def process_pdf_files():
    reports = get_pdf_files_scheduled_for_processing()
    for report in reports:
        process_pdf_file(report)
    





from django.db import models

class CenterManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)

class Center(models.Model):
    objects = CenterManager()

    # import_id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=255)
    name_de = models.CharField(max_length=255, blank=True, null=True)
    name_en = models.CharField(max_length=255, blank=True, null=True)

    first_names = models.ManyToManyField(
        'FirstName',
        related_name='centers',
    )
    last_names = models.ManyToManyField('LastName', related_name='centers')

    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name